#ifndef __ITR_MARVIN_H
#define __ITR_MARVIN_H

/* for chip and chamber dimensions */
#include "ITR_silraw.h"

#ifdef __cplusplus
extern "C" {
#endif


#define MASK  0x7FFFFFFF   /* bit pattern for masking the sign bit   */
#define NDIM           5   /* spline fit: number of knots in spline    */
#define NINT    (NDIM-1)   /* spline fit: number of intervals          */
#define NMAT  (3*NDIM-2)   /* spline fit: number of elements in weight matrix */
#define NCHAN 128
#define NEVMONP      128   /* pedestal following                  */ 
#define NEVMONV      128   /* variance following                  */
#define NCELL        141   /* number of pipeline cells            */

/*-----------------------------------------------------------------------------
 * structures used by marvin
 *---------------------------------------------------------------------------*/

typedef struct {
  int   format;       /* sparse data format            */
  int   chamber;      /* detector chamber              */
  int   width;        /* physical cluster width        */
  int   pwi;          /* begin of physical cluster     */
  int   first;        /* first strip in (whole) cluster*/
  int   end;          /* last strip in (whole) cluster */
  float cog;          /* center of gravity of cluster  */
  float localmean;    /* local baseline for ITR        */
  float charge;       /* total cluster charge          */
  float variance;     /* sum of noise^2                */
  float sn;           /* signal/noise                  */
  float *digits;      /* pointer to array of digits    */
  float *variances;   /* pointer to array of variances */ 
} ITR_Marvin_Cluster_t;


typedef struct {
  int           format;   /* sparse data format                     */
  int           size;     /* number of bytes available in *data     */
  char          *next;    /* pointer to next free location          */
  char          *data;    /* pointer to data buffer                 */ 
} ITR_Marvin_Sevt_t;



typedef struct {
  int  major;      /* major number of the current event (e.g. run number)   */
  int  minor;      /* minor number of the current event (e.g. event number) */
  int  cellshort;  /* cell number of the event for short pipeline           */
  int  celllong;
  int  format;     /* data format used for sparse data                      */
  int  ncl;        /* number of clusters in sparse data                     */
} ITR_Marvin_Header_t;


typedef struct {
  int           chipid;     /* chip identifier                              */
  int           chamberid;  /* chamber identifier for this chip             */
  void          *chamber;   /* pointer to chamber (has to be declared void) */
  int           chipstatus; /* chip status (1=filled,-1=bad,0=otherwise)    */
  int           nchan;      /* number of readout channels                   */
  int           pipelen;    /* length of the pipeline                       */
  int           first;      /* geometrical number of first strip            */
  int           last;       /* geometrical number of last  strip            */
  int           *strip;     /* geometrical strip numbers                    */
  unsigned char *digi;      /* raw data                                     */
  float         *data;      /* processed data for clustering                */
  int           *status;    /* channel status 1=OK, 0=dead                  */
  int           ncellp;     /* number of cells for pedestals                */ 
  int           ncellr;     /* number of cells for raw noise                */
  int           ncells;     /* number of cells for subtracted noise         */
  int           nevtp;      /* event counter(s) for pedestals               */  
  int           nevtr;      /* event counter(s) for raw noise^2             */
  int           nevts;      /* event counter(s) for subtr. noise^2          */
  float         *peds;      /* pedestals                                    */
  float         *rvar;      /* raw variances                                */
  float         *svar;      /* subtracted variances                         */
  int           hcolno;     /* height of column number in this event        */
  float         hcolno_mean;/* mean height of column number for this chip   */
  int           *nevtpp;       /* event counters for cell dependent pedestals */ 
  int           nevap;
  int           nevar;
  int           nevas;
  float         chan[3*NCHAN]; /* channel data: chi2, data, svar */
  float         svinv[NCHAN];
  float         svsum[NCHAN]; 
  float         pval[NCHAN];
  float         psum[NCHAN];
  int           istat[NCHAN];
  int           cstat[NCELL];  /* status of pipeline cells                  */

} ITR_Marvin_Chip_t;


typedef struct {
  int               chamberid;            /* chamber-identifier                           */
  int               chamberstatus;        /* chamber status (1=filled,-1=bad,0=otherwise) */
  int               first;                /* first strip filled in current event          */
  int               last;                 /* last  strip filled in current event          */
  int               nchan;                /* # of channels for current chamber            */
  int               nchip;                /* # of chips for current chamber               */
  int               chambertype;          /* type of chamber (I, II or III)               */
  float             threshold;            /* common threshold for this chamber */
  ITR_Marvin_Chip_t *chip[DIM_NCHIPCHAMBER]; /* pointer array to chips of chamber         */
  float             **data;               /* pointer array to data                        */
  float             **dvar;               /* pointer array to subtracted noise^2          */
} ITR_Marvin_Chamber_t;


typedef struct {
  ITR_Marvin_Header_t    header;                /* marvin header        */
  int                    nchamber;              /* # of chambers        */
  int                    nchip;                 /* # of chips           */
  ITR_Marvin_Chamber_t   *chamber[DIM_NCHAMBER];/* pointers to chambers */
  ITR_Marvin_Chip_t      *chip[DIM_NCHIP];      /* pointers to chips    */
} ITR_Marvin_Event_t;


typedef struct{
  IOL_MNAME   module[IOL_MLEN];
  IOL_SIZE    size; 
  IOL_STRING  version;        /* current version of marvin                 */
  IOL_INT     use_ITR_marvin; /* skip marvin_event if use_marvin==0        */
  IOL_STRING  pathsd;         /* path for sparse data file                 */
  IOL_INT     nevmon;         /* number of events to be kept in monitoring */
  IOL_FLOAT   seedcut;        /* seed-cut for VDS-clustering               */
  IOL_FLOAT   chi2cut;        /* chi2-cut for VDS-cluster quality          */
  IOL_FLOAT   smooth_cut;     /* cut on 'smoothness' of data               */
  IOL_FLOAT   flat_cut;       /* cut on 'flatness' of data                 */
  IOL_INT     sdformat;       /* sparse data format                        */
  IOL_INT     sdtype;         /* sparse data type (0=old,1..3=DAQ)         */
  IOL_INT     baseline;       /* degree of polynomials in baseline fit     */
  IOL_INT     nbaseline;      /* number of basline fits per chip           */
  IOL_INT     ncellshortp;    /* number of cells for pedestals             */
  IOL_INT     ncellshortr;    /* number of cells for raw noise^2           */
  IOL_INT     ncellshorts;    /* number of cells for subtracted noise^2    */
  IOL_INT     ncelllongp;     /* the same for the long pipeline            */
  IOL_INT     ncelllongr;     /* ""                                        */
  IOL_INT     ncelllongs;     /* ""                                        */
  IOL_INT     pedUpdateRate;  /* update rate for pedestal following        */
  IOL_INT     rvarUpdateRate; /* update rate for raw noise^2 following     */
  IOL_INT     svarUpdateRate; /* update rate for subtr. noise^2 following  */
  IOL_INT     reportRate;     /* report rate from marvin event             */
  IOL_INT     dumpRaw;        /* dump raw data from silraw                 */
  IOL_INT     dumpClusters;   /* dump cluster information to screen        */
  IOL_INT     debug_mode;     /* debug mode                                */
  /*VB: next 6 values for ITR */
  IOL_FLOAT   pulseHeight;    /* cut on max. pulseheight                   */
  IOL_FLOAT   minTotalCharge; /* require minimal total pulseheight for clusters */
  IOL_FLOAT   maxCluSize;     /* cut on max. custerwidth                   */
  IOL_FLOAT   relSTEIG;       /* maxSignal/Clusize                         */
  IOL_FLOAT   minDiff;        /* min ADC diff for begin of pulse search    */
  IOL_FLOAT   LocalMeanSize;  /* # channels for computation of baseline    */
  IOL_FLOAT   MaxPositiveCh;  /* ... for Noise Cut                         */
  IOL_FLOAT   CutRMS;         /* ""                                        */
  IOL_FLOAT   diffrile;       /* cut for right-left asymmetry              */
  IOL_INT     clusterfinder;  /* selection of clusterfinder                */
  IOL_INT     gaincorrection; /* correct for gain of electronic chain ?    */
  IOL_FLOAT   gc_MinHeight;   /* min. height of column number to participate*/
  IOL_INT     dynamic_threshold; /* if set, use pulseHeight*chamber_sigma as threshold */
  IOL_FLOAT   ClRat;          /* cluster Ratio ? for AL's cluster finder   */
  IOL_FLOAT   hotcut;         /* for detection of hot channels             */
  IOL_FLOAT   svar_maxSignal; /* cut for first determination of subtracted noise */
} ITR_Marvin_IOL_t;


/*-----------------------------------------------------------------------------
 * macros used by marvin
 *-----------------------------------------------------------------------------
  Sparse event input 
  ------------------
  The following set of macros allows to read in char, int or float arrays
  into a Marvin_Sevt_t-structure. The macros check for overflow and also
  implement byte swap if needed.
 *---------------------------------------------------------------------------*/

#define SEVT_BUFFER_FULL   1
#define SEVT_READ_FAILED   2
#define SEVT_FILE_CORRUPT  3

#define SEVT_IFREE  ((sevt->size - (sevt->next - sevt->data))/sizeof(int)) 
#define SEVT_FFREE  ((sevt->size - (sevt->next - sevt->data))/sizeof(float)) 
#define SEVT_CFREE  ((sevt->size - (sevt->next - sevt->data))/sizeof(char)) 

#define SEVT_IREAD(file, n)  fread(sevt->next, sizeof(int)  , n, file) 
#define SEVT_FREAD(file, n)  fread(sevt->next, sizeof(float), n, file) 
#define SEVT_CREAD(file, n)  fread(sevt->next, sizeof(char) , n, file) 

#define SEVT_IINPUT(file, n) {\
  if( SEVT_IFREE          < (n) ) return SEVT_BUFFER_FULL;\
  if( SEVT_IREAD(file, n) < (n) ) return SEVT_READ_FAILED;\
  sevt->next += (n)*sizeof(int);\
}

#define SEVT_FINPUT(file, n) {\
  if( SEVT_FFREE          < (n) ) return SEVT_BUFFER_FULL;\
  if( SEVT_FREAD(file, n) < (n) ) return SEVT_READ_FAILED;\
  sevt->next += (n)*sizeof(float);\
}

#define SEVT_CINPUT(file, n) {\
  if( SEVT_CFREE          < (n) ) return SEVT_BUFFER_FULL;\
  if( SEVT_CREAD(file, n) < (n) ) return SEVT_READ_FAILED;\
  sevt->next += (n)*sizeof(char);\
}

/*-----------------------------------------------------------------------------
 * prototypes 
 *---------------------------------------------------------------------------*/

/* run steering */
int ITR_marvin_param_init() ;
  /*int ITR_marvin_startprogram();*/
int ITR_marvin_startrun() ;
int ITR_marvin_init() ;
int ITR_marvin_event();
int ITR_marvin_endrun();
int ITR_marvin_endprogram();

/* access to internal structures */
ITR_Marvin_Sevt_t *ITR_marvin_fetch_sevt();
void              ITR_marvin_declare_sevt( ITR_Marvin_Sevt_t *sevt );

ITR_Marvin_Event_t *ITR_marvin_fetch_event();
void               ITR_marvin_declare_event( ITR_Marvin_Event_t *marvin_evt );

ITR_Marvin_Chip_t    *ITR_marvin_findChip( int chipid );
ITR_Marvin_Chamber_t *ITR_marvin_findChamber( int chamberid );

/* process one chip */
int  ITR_marvin_processChip     ( ITR_Marvin_Chip_t *chip, int cell );
int ITR_marvin_subtractPedestals( ITR_Marvin_Chip_t *chip, float *peds );
int ITR_marvin_followPedestals  ( ITR_Marvin_Chip_t *chip,
			      float *peds, int nevmon, int nevtp );
int ITR_marvin_doPedestals      ( ITR_Marvin_Chip_t *chip,
			      float *peds, int nevmon, int nevtp );
int ITR_marvin_followRawNoise   ( ITR_Marvin_Chip_t *chip,
			      float *rvar, int nevmon, int nevtr );
int ITR_marvin_followSubtractedNoise( ITR_Marvin_Chip_t *chip,
				  float *svar, int nevmon, int nevts );
int ITR_marvin_smoothCut        ( ITR_Marvin_Chip_t *chip,
			      float *svar, int nevmon, int nevts );
int ITR_marvin_flatCut          ( ITR_Marvin_Chip_t *chip,
			      float *svar, int nevmon, int nevts );
int ITR_marvin_subtractBaseline      ( int *status, float *y, int ny, int degree );
int ITR_marvin_subtractBaseline_lin  ( int *status, float *y, int ny );
int ITR_marvin_subtractBaseline_const( int *status, float *y, int ny );

/* initialization procedures */
int     ITR_marvin_readSetup  ();
int     ITR_marvin_cleanSetup (ITR_Marvin_Event_t *marvin_evt);
int     ITR_marvin_readMoni   (char *file );

/* help functions */
int     ITR_marvin_getChannelstatus(int chamberid, int *status);

/* clusterfinders */
  int     ITR_marvin_getNegativeClusters  (ITR_Marvin_Chamber_t *chamber, int cell);
  int     ITR_marvin_getITRClusters       (ITR_Marvin_Chamber_t *chamber, int cell);
  int     ITR_marvin_getSparseITRClusters (ITR_Marvin_Chamber_t *chamber, int cell);

/* output routines */
void    ITR_marvin_dumpClusters();
void    ITR_marvin_dumpRaw     ( int level );
void    ITR_marvin_dumpChamber ( ITR_Marvin_Chamber_t *chamber );
int     ITR_marvin_outputChip  ( ITR_Marvin_Chip_t *chip, FILE *mon_file ); 

void    ITR_marvin_swap        ( void *c, int n ); 

/* sevt routines */
int                  ITR_marvin_packSevt( int chamberid, int pwi, int start, int end,
			     float **d, float **v, int width, float *cog, 
			     float *charge, float *localmean );
int                  ITR_marvin_readSevt(ITR_Marvin_Sevt_t *sevt, FILE * sfile);
int                  ITR_marvin_writeSevt( FILE *sdfile );
ITR_Marvin_Header_t  *ITR_marvin_sevtHeader (char *data);
ITR_Marvin_Cluster_t *ITR_marvin_sevtCluster(char *data, int chamber);

/* correction functions for electronic gain */
int   ITR_marvin_followColumnNumber (ITR_Marvin_Chip_t *chip, int nevmon, int nevtp);
float ITR_marvin_computeMeanColNo(void);
int   ITR_marvin_norm_gain(ITR_Marvin_Chip_t *chip, ITR_Marvin_Chamber_t *chamber);
int ITR_marvin_calculate_threshold( ITR_Marvin_Chamber_t *chamber, int nevmon, int nevts );
/* other helper routines */
int ITR_marvin_detect_hotchannel( ITR_Marvin_Chip_t *chip, float *rvar, int nevmon, int nevtr );
int ITR_marvin_setdeadChannels( ITR_Marvin_Chip_t *chip );
int ITR_marvin_update_db(ITR_Marvin_Chip_t *chip) ;


int ITR_marvin_subtractSpline(int *istat, float *y, float CUT, int *outlier);
void ITR_marvin_solveSpline(float *saa, float *sab, float *sbb, float *sy);


#ifdef __cplusplus
}
#endif

#endif




