int ITR_silraw_check_event()
/*-----------------------------------------------------------------------------
 * check/repair the event integrity
 * ================================
 * For all chips the column number read out from the HELIX is checked,
 * using as input the offset provided in ITR_silraw_iol.datavalid_offset.
 * In a first round only chips with id=0 are examined.
 * In repair mode the validity of the assumed offset is checked by the 
 * program, and in case it fails the test it is overwritten by an 
 * alternative offset-hypothesis. In non-repair mode the given offset 
 * is accepted unconditionally. Using the offset, the column number 
 * of all id=0 chips is determined. The most frequent number is then
 * taken as the nominal one. In a second iteration over all chips 
 * the column number obtained by using the input offset is compared
 * with the nominal one. In case of discrepancies the repair mode 
 * searches for alternative offsets that reproduce the nominal column
 * number. If the nominal column number is not found, then the chip 
 * is flagged as "bad": chip->flag = 1, otherwise the data pointer 
 * shifted by the proper offset. Finally also the nominal cell number
 * is put into the ITR_silraw header structure. 
 * ... and this all for helixes with the long and the short pipeline
 *
 * If the pipelinelength is false the chip is flagged as "error with 
 * pipelen": chip->flag = 2. Often this is the case when the chamber 
 * not exist but is in the database!   
 *
 * In case any of the ITR_silraw debug-flags is on some information 
 * concerning cell numbers and offsets is printed before return.
 *
 * ITR_silraw_check_event() should be called before calling marvin.
 *
 * Author: Michael Schmelling / 3-Sep-1998
 *---------------------------------------------------------------------------*/
{
  int           i, j, failed, nfail;
  int           nhitshort, nhitlong, nmissshort, nmisslong, n2flag=0;
  int           rc, nchip, chipid, offset;
  int           icellshort, icelllong, cellnumshort, cellnumlong,  pipelen, flag;
  unsigned char *chipdata;
  static   int  cellshort[NCELLHELIXSHORT], celllong[NCELLHELIXLONG], offs[NOFFS];
  static   int  ccell[DIM_NCHIP], coffs[DIM_NCHIP];

  ITR_Silraw_Event_t *silraw;
  char *routine = "ITR_silraw_check_event";

  /*---------------------------------------------------------------------------*/

  /* get the number of chips that were read out */

  if(rc=ITR_silraw_get_nchip (&nchip )!= GRR_OK) return rc;
  if(nchip==0) 
    return GRR_FISHY;
  
  /* get ITR_silraw event */
  silraw = ITR_silraw_fetch_event();
  if ( !silraw )
    return GRR_FISHY;

  /* initialize the cell count */ /*VB*/
  for(i=0; i<NCELLHELIXSHORT; i++) cellshort[i] = 0;
  for(i=0; i<NCELLHELIXLONG;  i++) celllong[i]  = 0;


  /*
   *  deal with the non-repair mode:
   *
   *   take offset as given in iol
   *   determine all cell numbers
   *   flag all chips that don't conform to the majority
   *   fill cell number in ITR_silraw-header
   *   maybe do some debug prints and return
   */

  if( !ITR_silraw_iol.repair_mode.value ) {

   
    /* find and count columnnumbers */
    for(i=0; i<nchip; i++) {
      chipdata = silraw->chip[i].data;
      pipelen  = silraw->chip[i].pipelen;
      flag     = silraw->chip[i].flag;
      /*VB*/
       if (pipelen == NCELLHELIXSHORT){
	icellshort = ITR_silraw_fit_cell(chipdata + 128);
	ccell[i]   = icellshort;
	cellshort[icellshort]++;
      }
      else if (pipelen == NCELLHELIXLONG){
	icelllong = ITR_silraw_fit_cell(chipdata + 128); 
	ccell[i]  = icelllong;
	celllong[icelllong]++;
      }
      else {
	if(ITR_silraw_iol.debug_mode.value)
	  aframe_mess(routine, GRR_SERROR, "chip: 0x%x, false pipelinelength %d",
		      silraw->chip[i].chipid, pipelen);
	silraw->chip[i].flag = 2; /*VB: this means false pipeline, 
				    often in this case the chamber does not exist*/
	n2flag++;
      }
    } /*end for(...nchip...)-loop*/
    

    /* find most frequent columnnumber */
    cellnumshort = cellnumlong = 0;
    /*VB:*/
    for(i=0;i<NCELLHELIXSHORT; i++) {
      if(cellshort[i]>cellshort[cellnumshort]) cellnumshort = i;
    }
    for(i=0;i<NCELLHELIXLONG;i++){
      if(celllong[i]>celllong[cellnumlong]) cellnumlong = i;
    }
    nhitshort = nhitlong = nmissshort = nmisslong = 0;
    for(i=0; i<nchip; i++) {
      pipelen  = silraw->chip[i].pipelen;
      flag     = silraw->chip[i].flag;
      if (pipelen == NCELLHELIXSHORT){
	if(ccell[i] == cellnumshort) {
	  nhitshort++;
	} else {
	  nmissshort++;
	  silraw->chip[i].flag = 1;
	}
      }
      else if (pipelen == NCELLHELIXLONG){
	if(ccell[i] == cellnumlong) {
	  nhitlong++;
	} else {
	  nmisslong++;
	  silraw->chip[i].flag = 1;
	}
      }
      else {
	if (flag != 2){
	  if(ITR_silraw_iol.debug_mode.value)
	    aframe_mess(routine, GRR_SINFO, "chip 0x%x, false pipelinelength %d",
			silraw->chip[i].chipid,pipelen);
	  silraw->chip[i].flag = 2; /*VB: this means false pipeline, 
		      often in this case the chamber does not exist*/
	  n2flag++;
	}
      }
    } 

    silraw->header.cellshort = cellnumshort;
    silraw->header.celllong  = cellnumlong;
    
    if( ITR_silraw_iol.dump_header.value ||
        ITR_silraw_iol.dump_chips.value  ||
        ITR_silraw_iol.dump_feds.value   ||
        ITR_silraw_iol.debug_mode.value    ) { 
      printf("*** event check performed in non-repair mode ***\n");
      printf("cell number for short pipeline is           %4d \n",cellnumshort);
      /* printf("nominal offset from iol  %4d \n",
              ITR_silraw_iol.datavalid_offset.value);*/
      printf("CORRECT offset used for  %4d chips\n",nhitshort);   
      printf("WRONG   offset used for  %4d chips\n",nmissshort);
      if(nmissshort>nhitshort)
      printf("---> using repair mode recommended <--- \n\n");  
      printf("cell number for long pipeline is           %4d \n",cellnumlong);
      /* printf("nominal offset from iol  %4d \n",
              ITR_silraw_iol.datavalid_offset.value);*/
      printf("CORRECT offset used for  %4d chips\n",nhitlong);   
      printf("WRONG   offset used for  %4d chips\n",nmisslong);
      printf("number of chips with false pipeline: %4d \n",n2flag);
      printf("          (exist these chambers ??)\n");
      if(nmisslong>nhitlong)
      printf("---> using repair mode recommended <--- \n");  
    }
  
    return GRR_OK;
  }


  /*
   *  deal with the repair mode: 
   *   
   *   first use all chips with id=0
   *   verify the offset
   *   determine cell numbers
   *   accept majority as the nominal cell number
   *   then loop over all chips and if needed try to adjust the offset
   *   flag chips that cannot be made to conform to the majority
   *   fill cell number in ITR_silraw-header
   *   maybe do some debug prints and return
   */

  /* 
   *  loop over all chips and get nominal cell number from chips with id=0 
   */

  /* search right offset at the beginning of data */
  for(i=0; i<nchip; i++) {
    offset   = ITR_silraw_iol.datavalid_offset.value;
    chipdata = silraw->chip[i].data - offset;
    chipid   = silraw->chip[i].chipid & 0x00F0;
    pipelen  = silraw->chip[i].pipelen;
    flag     = silraw->chip[i].flag;
    if(chipid==0 && offset!=0) {
      if(ITR_silraw_check_offset(chipdata, offset)) {
	for(j=1; j<NOFFS; j++) {
          if(!ITR_silraw_check_offset(chipdata, j)) {
            offset = j; 
            break;
          } 
        }
      }
    }
    /* find and count columnnumbers */
    if(chipid == 0) {
      if (pipelen == NCELLHELIXSHORT){
	icellshort  = ITR_silraw_fit_cell(chipdata + offset + 128);
	ccell[i] = icellshort;
	coffs[i] = offset;
	cellshort[icellshort]++;
      }
      else if (pipelen == NCELLHELIXLONG){
	icelllong  = ITR_silraw_fit_cell(chipdata + offset + 128);
	ccell[i] = icelllong;
	coffs[i] = offset;
	celllong[icelllong]++;
      }
      else {
	if(ITR_silraw_iol.debug_mode.value)
	  aframe_mess (routine, GRR_SERROR, "chip 0x%x, false pipelinelength %d", 
		       chipid,pipelen);
	silraw->chip[i].flag = 2; /*VB: this means false pipeline, 
		    often in this case the chamber does not exist*/
	n2flag++;
      }
    }
  }  /*end for(... nchip...)-loop*/

  /* find most frequent columnnumber */
  cellnumshort = cellnumlong =  0;
  for(i=0; i<NCELLHELIXSHORT; i++) {
    if(cellshort[i]>cellshort[cellnumshort]) cellnumshort = i;
  }
  for(i=0; i<NCELLHELIXLONG; i++){
    if(celllong[i]>celllong[cellnumlong])    cellnumlong = i;
  }

  /*
   *  loop over all chips and try to find the correct cell number
   */

  
  for(i=0; i<NOFFS; i++) offs[i]=0;  /* zero offset counters   */
  nfail = nhitshort = nhitlong = nmissshort = nmisslong = 0;  /* zero success/failure statistics */

  for(i=0; i<nchip; i++) {
    failed = 1;
    chipdata = silraw->chip[i].data - ITR_silraw_iol.datavalid_offset.value;
    chipid   = silraw->chip[i].chipid & 0x00F0;
    pipelen  = silraw->chip[i].pipelen;
    if(pipelen == NCELLHELIXSHORT){ 
      if(chipid==0 && ccell[i]==cellnumshort) {
	offset = coffs[i];
	failed = 0;
	goto DONE;
      } else {
	offset = ITR_silraw_iol.datavalid_offset.value;
	icellshort   = ITR_silraw_fit_cell(chipdata + offset + 128);
	if(icellshort == cellnumshort) {
	  failed = 0;
	  goto DONE; 
	} else {
	  for(j=1; j<NOFFS; j++) {
	    icellshort = ITR_silraw_fit_cell(chipdata + j + 128);
	    if(icellshort == cellnumshort) {
	      offset = j;
	      failed = 0;
	      goto DONE; 
	    }
	  } 
	}       
      }
    }
    else if (pipelen == NCELLHELIXLONG){
      if(chipid==0 && ccell[i]==cellnumlong) {
	offset = coffs[i];
	failed = 0;
	goto DONE;
      } else {
	offset = ITR_silraw_iol.datavalid_offset.value;
	icelllong   = ITR_silraw_fit_cell(chipdata + offset + 128);
	if(icelllong == cellnumlong) {
	  failed = 0;
	  goto DONE; 
	} else {
	  for(j=1; j<NOFFS; j++) {
	    icelllong = ITR_silraw_fit_cell(chipdata + j + 128);
	    if(icelllong == cellnumlong) {
	      offset = j;
	      failed = 0;
	      goto DONE; 
	    }
	  } 
	}       
      } 
    }
    else { 
      if (flag != 2)
	{
	  if(ITR_silraw_iol.debug_mode.value)
	    aframe_mess(routine, GRR_SERROR, "chip: 0x%x, false pipelinelength %d\n", 
			chipid, pipelen);
	  silraw->chip[i].flag = 2; /*VB: this means false pipeline, 
		      often in this case the chamber does not exist*/
	  n2flag++;
	}
      failed = 0;
      goto DONE;
    }
    
    DONE:
    if(failed) {
      if (silraw->chip[i].flag != 2)
	silraw->chip[i].flag = 1;
      nfail++;
    } else {
      offs[offset]++;  /* count offsets */
      silraw->chip[i].data += offset - ITR_silraw_iol.datavalid_offset.value; /*new pointerposition*/
      if(offset == ITR_silraw_iol.datavalid_offset.value) {
	if (pipelen == NCELLHELIXSHORT) nhitshort++;
	else nhitlong++;
      } else {
	if(pipelen == NCELLHELIXSHORT) nmissshort++;
	else nmisslong++;
      }
    }
  } /*end for(...nchip...)-loop*/

  /* fill the pipeline column number */

  silraw->header.cellshort = cellnumshort;
  silraw->header.celllong  = cellnumlong;

  /* print diagnostics if any debugging is switched on */

  if( ITR_silraw_iol.dump_header.value ||
      ITR_silraw_iol.dump_chips.value  ||
      ITR_silraw_iol.dump_feds.value   ||
      ITR_silraw_iol.debug_mode.value  ||
      (ITR_silraw_iol.cell_check_output.value == 2 )) { 

      /* determine also the preferred offset */  
    
      offset = 0;
      for(i=0; i<NOFFS; i++) {
         if(offs[i]>offs[offset]) offset = i;
      }
      
      printf("*** event check performed in REPAIR mode ***\n");     
      printf("current cell number for short pipeline is             : %4d \n",cellnumshort); 
      printf("preferred offset is                : %4d \n",offset);
      /*  printf("nominal offset taken from iol is   : %4d \n",
              ITR_silraw_iol.datavalid_offset.value);*/
      printf("number of chips with nominal offset: %4d \n",nhitshort);
      printf("                 alternative offset: %4d \n",nmissshort);
      printf("current cell number for long pipeline is             : %4d \n",cellnumlong); 
      printf("number of chips with nominal offset: %4d \n",nhitlong);
      printf("                 alternative offset: %4d \n",nmisslong);
      printf("        offset determination failed: %4d \n",nfail);
      printf("number of chips with false pipeline: %4d \n",n2flag);
      printf("             (exist these chambers??)\n");
  }

   return GRR_OK;
}


/*-----------------------------------------------------------------------------
 * check cell number consistency for all chips
 * ===========================================
 * 
 * Author: Rainer Wanke / 1-Oct-1998
 * long and short pipeline: VB July 1999
 *---------------------------------------------------------------------------*/
int ITR_silraw_check_cell_consistency() 
{
  int    ifed, icell, ifail,i,nchip;
  int    linkfail;
  int    cell1[DIM_NFED], cell2[DIM_NFED], pipelen, rc;
  /*VB  appearance count of each cell in event*/
  int    cellcountshort[NCELLHELIXSHORT],cellcountlong[NCELLHELIXLONG];
  int    ncellmax, bestcell;
  int    bestcellshort,bestcelllong; /*VB*/
  int    ncellmaxshort,ncellmaxlong; /*VB*/

  static int shit_happens       = 0;
  static int shit_happens_count = 0;
  static int shit_happens_event = 0;

  ITR_Silraw_Event_t *silraw;

  time_t mytime;

  char *routine = "ITR_silraw_check_cell_consistency";

  /* ---------------------------------------------------------- */

  /* fetch ITR_Silraw_Event_t */
  silraw = ITR_silraw_fetch_event();
  if ( !silraw )
    return GRR_FISHY;

  /*VB*/
  if(rc=ITR_silraw_get_nchip (&nchip )!= GRR_OK) return rc;
  if(nchip==0) return GRR_FISHY;

  /* initialization */
  linkfail = 0;
  for ( icell=0; icell<NCELLHELIXMAX; icell++ ){
    cellcountshort[icell] = 0; cellcountlong[icell] = 0;
  }

  /* print event number */
  if ( ITR_silraw_evno < 15 && ITR_silraw_iol.cell_check_output.value ) {
    aframe_mess(routine,GRR_SINFO,"----------------------------------");
    aframe_mess(routine,GRR_SINFO,"           EVENT %d", ITR_silraw_evno );
    aframe_mess(routine,GRR_SINFO,"----------------------------------");
  }

 
  /* loop over all feds (=links) */  
  for ( ifed=0; ifed<silraw->nfed; ifed++) {

    /*VB*/
    /*fill fed pipelen*/
  
    for(i=0;i<nchip;i++){
      if ((silraw->chip[i].fedid & 0xff0f) == (silraw->fed[ifed].fedid & 0xff0f) ){ 
	silraw->fed[ifed].pipelen = silraw->chip[i].pipelen;
	/*if(ITR_silraw_iol.debug_mode.value)
	  aframe_mess(routine,GRR_SINFO,"Fed 0x%x Pipeline %d",
	  silraw->fed[ifed].fedid,silraw->fed[ifed].pipelen);
	*/
      }
    }


    /* fill fedid and cell number arrays */    
    if ( ITR_silraw_evno == 1 ) {
      ITR_silraw_chipid[2*ifed]   = ( silraw->fed[ifed].fedid & 0xff0f );
      ITR_silraw_chipid[2*ifed+1] = ( silraw->fed[ifed].fedid & 0xff0f ) | 0x10;
    }
    cell1[ifed] = ITR_silraw_fit_cell( silraw->fed[ifed].data
				   + silraw->fed[ifed].offset + 128 );
    cell2[ifed] = ITR_silraw_fit_cell( silraw->fed[ifed].data
				   + silraw->fed[ifed].offset + 128 + 136 );
    if ( ITR_silraw_evno < 15 && ITR_silraw_iol.cell_check_output.value == 2 )
      aframe_mess(routine,GRR_SINFO, "link=%4d, fed=0x%x: cells are %3d %3d",
		  ifed, silraw->fed[ifed].fedid, cell1[ifed], cell2[ifed]);

    /* check and count cell numbers */
    /*VB: */
    if ( silraw->fed[ifed].pipelen == NCELLHELIXSHORT ){
      if ( cell1[ifed] < NCELLHELIXSHORT )
	cellcountshort[cell1[ifed]]++;
      else
	aframe_mess(routine,GRR_SINFO, "Short cell # %d out of range, link=%d, fed=0x%x, 1.helix",
		    cell1[ifed], ifed, silraw->fed[ifed].fedid ); 
      
      if ( cell2[ifed] < NCELLHELIXSHORT )
	cellcountshort[cell2[ifed]]++;
      else
	aframe_mess(routine,GRR_SINFO,"Short cell # %d out of range, link=%d, fed=0x%x, 2.helix",
		    cell2[ifed], ifed, silraw->fed[ifed].fedid ); 
    }
    
    else if ( silraw->fed[ifed].pipelen == NCELLHELIXLONG ){
      if ( cell1[ifed] < NCELLHELIXLONG )
	cellcountlong[cell1[ifed]]++;
      else
	aframe_mess(routine,GRR_SINFO, "Long cell # %d out of range, link=%d, fed=0x%x, 1.helix",
		    cell1[ifed], ifed, silraw->fed[ifed].fedid ); 
      
      if ( cell2[ifed] < NCELLHELIXLONG )
	cellcountlong[cell2[ifed]]++;
      else
	aframe_mess(routine,GRR_SINFO,"Long cell # %d out of range, link=%d, fed=0x%x, 2.helix",
		    cell2[ifed], ifed, silraw->fed[ifed].fedid ); 
    }
    
    else 
      {if(ITR_silraw_iol.debug_mode.value)
      aframe_mess (routine, GRR_SWARNING, "pipelinelength %d from FED 0x%x not valid!", 
		      silraw->fed[ifed].pipelen, silraw->fed[ifed].fedid);
    /*VB: FED markieren??Kammer vorhanden??*/
      }
  }/*end of fed-loop*/


  /*VB: aufgeteilt...*/
  /* Now get most preferred cell # */
  ncellmaxshort = 0;  ncellmaxlong = 0;
  bestcellshort = -1; bestcelllong = -1;
  for (icell=0; icell<NCELLHELIXSHORT;icell++){
    if ( cellcountshort[icell] > ncellmaxshort ) { 
      bestcellshort = icell; 
      ncellmaxshort = cellcountshort[icell]; 
    } 
  }
  for (icell=0; icell<NCELLHELIXLONG;icell++){
    if ( cellcountlong[icell] > ncellmaxlong ) { 
      bestcelllong = icell; 
      ncellmaxlong = cellcountlong[icell]; 
    } 
  }


  /*VB: aufgeteilt*/
  /* Loop over cell # arrays to uncover the traitors */
  for ( ifed=0; ifed<silraw->nfed; ifed++ ) {
    if ( silraw->fed[ifed].pipelen == NCELLHELIXSHORT ){
      if ( cell1[ifed] != bestcellshort && cell2[ifed] != bestcellshort )
	linkfail++;
      if ( cell1[ifed] != bestcellshort && ITR_silraw_evno > 10 ) {
	ITR_silraw_chipfail[2*ifed]++;
	if ( ITR_silraw_iol.cell_check_output.value == 2 )
	  aframe_mess(routine,GRR_SINFO,"Evt %d: Chip 0x%4x: Short cell # %3d differs from common cell # %3d",
		      ITR_silraw_evno, ITR_silraw_chipid[2*ifed], cell1[ifed], bestcellshort );
      }
      if ( cell2[ifed] != bestcellshort && ITR_silraw_evno > 10 ) {
	ITR_silraw_chipfail[2*ifed+1]++;
	if ( ITR_silraw_iol.cell_check_output.value == 2 )
	  aframe_mess(routine,GRR_SINFO, "Evt %d: Chip 0x%4x: Short cell # %3d differs from common cell # %3d",
		      ITR_silraw_evno, ITR_silraw_chipid[2*ifed+1], cell2[ifed], bestcellshort );
      }
    }
    else if (silraw->fed[ifed].pipelen == NCELLHELIXLONG){
      if ( cell1[ifed] != bestcelllong && cell2[ifed] != bestcelllong )
	linkfail++;
      if ( cell1[ifed] != bestcelllong && ITR_silraw_evno > 10 ) {
	ITR_silraw_chipfail[2*ifed]++;
	if ( ITR_silraw_iol.cell_check_output.value == 2 )
	  aframe_mess(routine,GRR_SINFO,"Evt %d: Chip 0x%4x: Long cell # %3d differs from common cell # %3d",
		      ITR_silraw_evno, ITR_silraw_chipid[2*ifed], cell1[ifed], bestcelllong );
      }
      if ( cell2[ifed] != bestcelllong && ITR_silraw_evno > 10 ) {
	ITR_silraw_chipfail[2*ifed+1]++;
	if ( ITR_silraw_iol.cell_check_output.value == 2 )
	  aframe_mess(routine,GRR_SINFO, "Evt %d: Chip 0x%4x: Long cell # %3d differs from common cell # %3d",
		      ITR_silraw_evno, ITR_silraw_chipid[2*ifed+1], cell2[ifed], bestcelllong );
      }
    }
    else {
      if(ITR_silraw_iol.debug_mode.value)
      aframe_mess(routine,GRR_SINFO,"pipelinelength %d for FED 0x%x not valid!",
		     silraw->fed[ifed].pipelen,silraw->fed[ifed].fedid);
    }  
  } /*end of for (..ifed..)-loop*/ 

 
  /*VB: */
  /* check most preferred cell number against cell number found by ITR_silraw */
  if ( bestcellshort != silraw->header.cellshort && bestcellshort != -1 ) {
    ITR_silraw_cellfail_tot_short++;
    aframe_mess(routine,GRR_SINFO, "Evt %d: Cell(short) # %d differs from ITR_silraw cell # %d",
	 	ITR_silraw_evno, bestcellshort, silraw->header.cellshort );
  }

  if ( bestcelllong != silraw->header.celllong && bestcelllong != -1 ) {
    ITR_silraw_cellfail_tot_long++;
    aframe_mess(routine,GRR_SINFO, "Evt %d: Cell(long) # %d differs from ITR_silraw cell # %d",
	 	ITR_silraw_evno, bestcelllong, silraw->header.celllong );
  }

  /* Count failure rates since event 10 */
  if ( ITR_silraw_evno >= 10 ) {
    ITR_silraw_linkfail_tot += linkfail;
    if ( linkfail > 0) 
      ITR_silraw_evfail_tot++;
  }

  /* Check, if Helixes have become bad :
   * Need 5 consecutive bad events => then dump out event, but only one */
  if ( linkfail > 70 && ITR_silraw_evno > 10 && !shit_happens ) {
    if ( shit_happens_event == ITR_silraw_evno-1 ) 
      shit_happens_count++;
    if ( shit_happens_count >= 5 ) {
      shit_happens = 1;
      mytime       = time(NULL);
      aframe_mess(routine,GRR_SINFO, "----------------------------------");
      aframe_mess(routine,GRR_SINFO, " GOT THE PROBLEM IN EVENT %d", ITR_silraw_evno );
      aframe_mess(routine,GRR_SINFO, asctime(localtime(&mytime)) );
      aframe_mess(routine,GRR_SINFO, "----------------------------------");
      for ( ifed=0; ifed<silraw->nfed; ifed++)
	aframe_mess(routine,GRR_SINFO, "link=%4d, fed=0x%x: cells are %3d %3d",
		    ifed, silraw->fed[ifed].fedid, cell1[ifed], cell2[ifed]);
    }
    shit_happens_event = ITR_silraw_evno;
  }
  else
    shit_happens_count = 0;

  /*VB: aufgeteilt...*/
  /* print out results */
  if ( ITR_silraw_evno < 15 && ITR_silraw_iol.cell_check_output.value ) {
    aframe_mess(routine,GRR_SINFO,
		"Best (short pipeline) cell # is %d, appears %d (%d) times.",
		bestcellshort, ncellmaxshort, 2*silraw->nfed );
    aframe_mess(routine,GRR_SINFO,
		"Best (long pipeline) cell # is %d, appears %d (%d) times.",
		bestcelllong, ncellmaxlong, 2*silraw->nfed );
    aframe_mess(routine,GRR_SINFO,
		"%d links failed (both helix cell #'s differ)",
		linkfail );
  }
  if ( linkfail > 0 && ( ITR_silraw_evno < 10000 || ITR_silraw_evno%100 == 0 )
                    && ITR_silraw_iol.cell_check_output.value      )
    aframe_mess(routine,GRR_SINFO, "Evt=%d: asynch links=%d",
		ITR_silraw_evno, linkfail );
  if ( linkfail > 70 && ( ITR_silraw_evno < 10000 || ITR_silraw_evno%100 == 0 )
                     && !ITR_silraw_iol.cell_check_output.value      )
    aframe_mess(routine,GRR_SINFO, "Evt=%d: asynch links=%d",
		ITR_silraw_evno, linkfail );
  if ( ITR_silraw_evno >= 100 && ITR_silraw_evno%400 == 1 )
    aframe_mess(routine,GRR_SINFO,
		"Evt %d: %d links in %d events out of sync since evt 10",
		ITR_silraw_evno, ITR_silraw_linkfail_tot, ITR_silraw_evfail_tot );

  /* increase event number */
  ITR_silraw_evno++;

  return GRR_OK;
}


/*-----------------------------------------------------------------------------
 * check an assumed offset for digitizings
 * =======================================
 * Given a candidate offset (for chips with id=0) this function checks
 * whether the first offset bytes are essentially constant, followed
 * by a jump in the next digitizing.
 * 
 * Author: Michael Schmelling / 2-Sep-1998
 *---------------------------------------------------------------------------*/
int ITR_silraw_check_offset(unsigned char *data, int offset)
{
  int   i, ifail=0;
  float val, s1, s2;
/*---------------------------------------------------------------------------*/

  for(i=0; i<offset; i++) {
    val = (float)*(data+i);
    s1 += val;
    s2 += val*val;
  }
  s1 = s1/(float)offset;
  s2 = s2/(float)offset - s1*s1;
  
  val = (float)*(data+offset) - s1;
  if(s2>2.|| val*val<10.) ifail++; 

  return ifail;

}

